// BMP180, V. Ziemann, 170805
#include <Wire.h>
const int BMP=0x77;   // I2C address
int16_t ac1,ac2,ac3,b1,b2,mb,mc,md;
uint16_t ac4,ac5,ac6;
void I2Cwrite(uint8_t addr, uint8_t reg, uint8_t val) {
  Wire.beginTransmission(addr);  // address
  Wire.write(reg);               // register
  Wire.write(val);               // write value
  Wire.endTransmission(true); 
}
uint8_t I2Cread(uint8_t addr, uint8_t reg) {
  Wire.beginTransmission(addr); // address
  Wire.write(reg);              // register
  Wire.endTransmission(false);
  Wire.requestFrom(addr,1);      // request 1 byte
  return Wire.read();           // read and return
}
bool bmp180_init() {  //......................bmp180_init
  Wire.beginTransmission(BMP);  // read EEPROM
  Wire.write(0xAA);   // start register address
  Wire.endTransmission(false);
  if (Wire.requestFrom(BMP,22)) {
    ac1=Wire.read()<<8 | Wire.read();
    ac2=Wire.read()<<8 | Wire.read();
    ac3=Wire.read()<<8 | Wire.read();
    ac4=Wire.read()<<8 | Wire.read();
    ac5=Wire.read()<<8 | Wire.read();
    ac6=Wire.read()<<8 | Wire.read();
    b1=Wire.read()<<8 | Wire.read();
    b2=Wire.read()<<8 | Wire.read();
    mb=Wire.read()<<8 | Wire.read();
    mc=Wire.read()<<8 | Wire.read();
    md=Wire.read()<<8 | Wire.read();
    Wire.endTransmission();
    return true; 
  } 
  return false;
}
float bmp180_getdata(float *T) {  //.......bmp180_getdata
  float ut,x1,x2,x3,b3,b4,b5,b6,b7,tt,p;
  uint16_t up;
  I2Cwrite(BMP,0xF4,0x2E);  // request temperature first
  delay(5);
  ut=(float)(I2Cread(BMP,0xF6) << 8 | I2Cread(BMP,0xF7));
  x1=(ut-ac6)*ac5/32768.0;
  x2=mc*2048.0/(x1+md);
  b5=x1+x2;
  *T=(b5+8.0)/160.0;         
  I2Cwrite(BMP,0xF4,0x34);  // request pressure measurement
  delay(5);                 // with oss=0, wait 5 ms, table 8 
  up=I2Cread(BMP,0xF6)<<8 | I2Cread(BMP,0xF7);
  b6=b5-4000;
  x1=(b2*(b6*b6/4096.0))/2048.0;
  x2=ac2*b6/2048.0;
  x3=x1+x2;
  b3=((((long)ac1*4+x3))+2)/4;
  x1=ac3*b6/8192;
  x2=(b1*(b6*b6/4096))/65536;
  x3=((x1+x2)+2)/4;
  b4=ac4*(x3+32768)/32768;
  b7=(up-b3)*(50000);
  if (b7< 0x80000000) {p=b7*2/b4;} else {p=(b7/b4)*2;}
  x1=(p/256)*(p/256);
  x1=(x1*3038)/65536;
  x2=(-7357*p)/65536;
  p=p+(x1+x2+3791)/16;
  return 0.01*p;
}
void setup() {  //..................................setup
  Serial.begin (9600); while (!Serial) {;}
  Wire.begin();
  if (!bmp180_init()) Serial.println("No sensor found!");
}
void loop() {  //....................................loop
  float pressure, temperature;
  pressure=bmp180_getdata(&temperature);
  Serial.print(temperature); Serial.print("\t"); 
  Serial.println(pressure);
  delay(2000);
}
