/* Simple socket server to serve barometric pressure, humidity, 
 * and temperature. Author: V. Ziemann, 170117
 */
#include <SFE_BMP180.h>
#include <Wire.h>
SFE_BMP180 pressure;
double P,TP,H,TH;
const int HYT=0x28;  // I2C address for HYT221
const char* ssid     = "messnetz";
const char* password = ".........";
const int port = 1137;
#include <ESP8266WiFi.h>
WiFiServer server(port);
void setup() {  //................................................setup
  pinMode(LED_BUILTIN,OUTPUT);
  digitalWrite(LED_BUILTIN,LOW);
  Serial.begin(115200);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.print("\nWiFi connected to "); Serial.println(ssid);
  Serial.print("Server IP address: "); Serial.println(WiFi.localIP());
  server.begin();          
  Serial.print("Server started on port "); Serial.println(port);
  digitalWrite(LED_BUILTIN,HIGH);
  if(!pressure.begin()) {Serial.println("Cannot init BPM180!");}
}
void loop() {  //................................................loop
  char line[30];
  float volt,temp;
  int t0;
  WiFiClient client = server.available();
  while (client) {
    digitalWrite(LED_BUILTIN,LOW);   
    while(!client.available()) {
      delay(5);
      if (!client.connected()) break;
    }
    client.readStringUntil('\n').toCharArray(line,30);
    Serial.print("Request: "); Serial.println(line);
    if (strstr(line,"TH?")==line) {     
      client.print("TH "); client.println(TH,2);
    } else if (strstr(line,"H?")==line) {
       int b1,b2,b3,b4,raw;
       Wire.beginTransmission(HYT);
       Wire.requestFrom(HYT,4); 
       delay(200);
       if (Wire.available()==4) {
         b1=Wire.read(); b2=Wire.read();  // humidity rawdata
         b3=Wire.read(); b4=Wire.read();  // temperature rawdata
         Wire.endTransmission();
       }     
       raw=(256*b1+b2) & 0x3FFF;      // humidity
       H=100.0*raw/16384.0;         
       raw=((256*b3+b4) & 0xFFFC)/4;   // temperature
       TH=165.0*raw/16384.0-40.0; 
       client.print("H "); client.println(H,2);
    } else if (strstr(line,"TP?")==line) {  
      client.print("TP "); client.println(TP,2);
    } else if (strstr(line,"P?")==line) {
      int is=pressure.startTemperature();
      if (is!=0) {
         delay(is);
         is=pressure.getTemperature(TP);
         if (is!=0) {
            is=pressure.startPressure(3);  // oversampling=3
            delay(is);
            is=pressure.getPressure(P,TP);
            if (is!=0) {
               Serial.print("BMP180: T="); Serial.print(TP,2); 
               Serial.print(" P="); Serial.println(P,2);
               client.print("P "); client.println(P,2);
            } else {
               Serial.println("Error: BPM180 getPressure failed");
            }
         } else {
           Serial.println("Error: BPM180 getTemperature failed");
         }    
       } else {
         Serial.println("Error: BPM180 startTemperature failed");
       } 
    } else if (strstr(line,"T?")==line) {
      temp=100*3.3*analogRead(0)/1023;
      client.print("T "); client.println(temp,1);
    } else {
      Serial.println("unknown command, disconnecting");
      client.stop();
    }
    client.flush();
  }
  digitalWrite(LED_BUILTIN,HIGH);
  yield();
}
